#!/bin/bash

if [ $(basename $0) = $(basename ${BASH_SOURCE}) ]; then # 単独で実行
    alone=1 
    while getopts Sg OPT
    do
      case $OPT in
        S) gs=2; echo "option: prepared voice"; ;;
        g) gs=1; echo "option: Google Speech"; ;;
        *) ;;
      esac
    done
    shift $((OPTIND - 1))
    if [ -e /usr/local/bin/gdate ]; then # macではgdate(brew install coreutils)があれば
        export DATECMD=/usr/local/bin/gdate         # dateにgdateを用いるように
    else
        export DATECMD=$(which date)
    fi
else
    alone=0
fi

googlespeech() {
    local IFS=+
    local s=$(xargs -0)
    mplayer -ao alsa -really-quiet -noconsolecontrols "http://translate.google.com/translate_tts?ie=UTF-8&client=tw-ob&q=${s}&tl=en" > /dev/null 2>&1
}

speech() {
    local voicedir=$(dirname $0)/voice
    local mp3="${voicedir}/$1.mp3"
    echo "mp3 file = ${mp3}"
    mplayer -ao alsa -really-quiet -noconsolecontrols ${mp3}
}

if [ ${gs:-0} -eq 1 -a -x /usr/bin/mplayer ]; then
    echo "[Info] \"Google text to speech\" is available."
    speak=2
    SPCMD="googlespeech"
elif [ ${gs:-0} -eq 2 -a -x /usr/bin/mplayer ]; then
    echo "[Info] \"prepared voice\" is available. ...voice data required"
    speak=2
    SPCMD="speech"
elif [ -x /usr/bin/say ] ; then
    unset gs
    echo "[Info] \"say\" is available. Yes, macOS."
    # available voice list : say -v ? | egrep 'en_'
    # Alex     ,en_US
    # Daniel   ,en_GB
    # Fred     ,en_US
    # Karen    ,en_AU
    # Moira    ,en_IE
    # Samantha ,en_US
    # Tessa    ,en_ZA
    # Veena    ,en_IN
    # Victoria ,en_US
    voice="Samantha"
    if [ ${DATECMD} = "/usr/local/bin/gdate" ]; then
        speak=2
    else
        speak=1
    fi
    SPCMD="/usr/bin/say"
elif [ -x /usr/bin/espeak ] ; then
    unset gs
    echo "[Info] \"espeak\" is available. Yes, Linux."
    fgrep "Raspbian" /etc/issue > /dev/null 2>&1
    if [ $? -eq 0 ]; then # Raspbian
        RPI=1
        if [ ! -x /usr/bin/jackd ]; then
            echo "[Warning] No \"jackd\" even on Rpi, no \"Speak\" functions!"
            speak=0
            [ ${alone} -eq 0 ] && return $(false)
            exit 1
        fi
        ps ax | fgrep "jackd" | fgrep -v "grep" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo 'starting "jackd"'
            jackd -d alsa -r 48000 -n 16 > /dev/null 2>&1 &
            jackd_pid=$!
            ps ax | egrep 'jackd' | egrep -v 'grep'
            sleep 1
        fi
    fi
    # available voice list : espeak --voices | awk '$2 ~ /^en/{print $4}
    #voice="default"
    voice="english-us"
    speak=2
    SPCMD="/usr/bin/espeak"
else
    echo "[Warning] No \"Speak\" functions!"
    speak=0
    [ ${alone} -eq 0 ] && return $(false)
    exit 1
fi

if [ ${debug:-0} -gt 0 ] ; then
    speaklog="/tmp/timesignal.log"
else
    speaklog="/dev/null"
    #speaklog="/dev/stdout"
fi
cat /dev/null > ${speaklog}
echo > ${speaklog}

now() {
    if [ "x$1" = "x" ]; then
        printf "It's $(${DATECMD} +"%_H") o'clock."
    elif [ "x$1" = "x0" ]; then
        printf "I think, It's $(${DATECMD} +"%_H:%_M")."
    elif [ ${speak} -gt 1 ]; then # might be linux only function
        printf "It's $(${DATECMD} --date "1 minutes" +"%_H") o'clock."
    fi
}

its() { # $1 null:現在時, 0:stand aline, 1:現在時+1分
    if [ "x${voice}" != "x" ] ; then # "say or espeak"
        now $1 | ${SPCMD} -v ${voice} >> ${speaklog} 2>&1 &
    elif [ ${SPCMD} = "googlespeech" ] ; then # "google speech"
        now $1 | ${SPCMD} >> ${speaklog} 2>&1 &
    else # "prepared voice"
        if [ ${speak} -gt 1 ]; then # might be linux only function
            ${SPCMD} $(${DATECMD} --date "1 minutes" +"%_H") >> ${speaklog} 2>&1 &
        else
            ${SPCMD} $(${DATECMD} +"%_H") >> ${speaklog} 2>&1 &
        fi
    fi
}

if [ "x${voice}" != "x" ]; then
    echo "I am ${voice}. I'll tell you the time." | ${SPCMD} -v ${voice} >> ${speaklog} 2>&1 
elif [ ${gs} -eq 1 ]; then
    echo "I am Google Text to Speech. I'll tell you the time." | ${SPCMD} >> ${speaklog} 2>&1 
else
    for i in 0 # {0..23}
    do
        ${SPCMD} ${i} >> ${speaklog} 2>&1
    done
fi

if [ ${alone} -eq 1 ]; then # 単独で実行
    unset alone
    now 0 ; echo
    its 0
    [ ${jackd_pid:-0} -ne 0 ] && kill ${jackd_pid}
else
    unset alone
    true
fi

